@@ -5,7 +5,7 @@
 
     The Anti-Lamenessing Engine is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
+    the Free Software Foundation; either version 3 of the License, or
     (at your option) any later version.
 
     The Anti-Lamenessing Engine is distributed in the hope that it will be useful,
@@ -51,7 +51,7 @@ class image : protected exposure::listener {
 protected:
 	unsigned int _dimx, _dimy, _depth;
 	point _offset;
-	char *name;
+	const char *name;
 	mutable exposure *_exp;
 	unsigned int bayer;
 private:
@@ -135,7 +135,7 @@ protected:
 
 public:
 	image (unsigned int dimy, unsigned int dimx, unsigned int depth, 
-			char *name = "anonymous", exposure *_exp = NULL,
+			const char *name = "anonymous", exposure *_exp = NULL,
 			unsigned int bayer = IMAGE_BAYER_NONE) {
 
 		assert (depth == 3);
@@ -159,6 +159,17 @@ public:
 		return bayer;
 	}
 
+	virtual char get_channels(int i, int j) const {
+		return 0x7;
+	}
+
+	double storage_size() const {
+		if (bayer != IMAGE_BAYER_NONE)
+			return _dimx * _dimy * sizeof(ale_real);
+
+		return 3 * _dimx * _dimy * sizeof(ale_real);
+	}
+
 	exposure &exp() const {
 		return *_exp;
 	}
@@ -192,9 +203,9 @@ public:
 
 	virtual const spixel &pix(unsigned int y, unsigned int x) const = 0;
 
-	virtual ale_real &chan(unsigned int y, unsigned int x, unsigned int k) = 0;
+	virtual ale_sreal &chan(unsigned int y, unsigned int x, unsigned int k) = 0;
 
-	virtual const ale_real &chan(unsigned int y, unsigned int x, unsigned int k) const = 0;
+	virtual const ale_sreal &chan(unsigned int y, unsigned int x, unsigned int k) const = 0;
 
 	virtual void set_pixel(unsigned int y, unsigned int x, spixel p) {
 		pix(y, x) = p;
@@ -204,6 +215,10 @@ public:
 		return ((const image *)this)->pix(y, x);
 	}
 
+	virtual pixel get_raw_pixel(unsigned int y, unsigned int x) const {
+		return ((const image *)this)->get_pixel(y, x);
+	}
+
 	ale_real maxval() const {
 		ale_real result = get_pixel(0, 0)[0];
 
@@ -239,8 +254,6 @@ public:
 	 */
 
 	pixel get_max_diff(unsigned int i, unsigned int j) const {
-		assert(i >= 0);
-		assert(j >= 0);
 		assert(i <= _dimy - 1);
 		assert(j <= _dimx - 1);
 
@@ -285,82 +298,6 @@ public:
 		return get_max_diff(i, j);
 	}
 
-	/*
-	 * Get a color value at a given position using bilinear interpolation between the
-	 * four nearest pixels.  Result values:
-	 *
-	 * result[0] == pixel value
-	 * result[1] == pixel confidence
-	 */
-	void get_bl(point x, pixel result[2]) const {
-
-		assert (x[0] >= 0);
-		assert (x[1] >= 0);
-		assert (x[0] <= _dimy - 1);
-		assert (x[1] <= _dimx - 1);
-
-		int lx = (int) floor(x[1]);
-		int hx = (int) floor(x[1]) + 1;
-		int ly = (int) floor(x[0]);
-		int hy = (int) floor(x[0]) + 1;
-
-		pixel neighbor[4];
-		ale_pos factor[4];
-
-		neighbor[0] = get_pixel(ly, lx);
-		neighbor[1] = get_pixel(hy % _dimy, lx);
-		neighbor[2] = get_pixel(hy % _dimy, hx % _dimx);
-		neighbor[3] = get_pixel(ly, hx % _dimx);
-
-		factor[0] = (hx - x[1]) * (hy - x[0]);
-		factor[1] = (hx - x[1]) * (x[0] - ly);
-		factor[2] = (x[1] - lx) * (x[0] - ly);
-		factor[3] = (x[1] - lx) * (hy - x[0]);
-
-		/*
-		 * Use bilinear interpolation for pixel value
-		 */
-
-		result[0] = pixel(0, 0, 0);
-
-		for (int n = 0; n < 4; n++)
-			result[0] += factor[n] * neighbor[n];
-
-#if 0
-		/*
-		 * Take the minimum confidence
-		 */
-
-		if (_exp) {
-			result[1] = _exp->confidence(neighbor[0]);
-
-			for (int n = 1; n < 4; n++)
-				if (factor[n] > 0) {
-					pixel confidence 
-						= _exp->confidence(neighbor[n]);
-
-					for (int k = 0; k < 3; k++)
-						if (confidence[k] < result[1][k])
-							result[1][k] = confidence[k];
-				}
-		} else {
-			result[1] = pixel(1, 1, 1);
-		}
-#else
-		/*
-		 * Use bilinear interpolation for confidence
-		 */
-
-		if  (_exp) {
-			result[1] = pixel(0, 0, 0);
-			for (int n = 0; n < 4; n++)
-				result[1] += factor[n] * _exp->confidence(neighbor[n]);
-		} else {
-			result[1] = pixel(1, 1, 1);
-		}
-#endif
-	}
-
 	int in_bounds(point x) const {
 		if (x[0] < 0
 		 || x[1] < 0
@@ -372,39 +309,111 @@ public:
 		return 1;
 	}
 
-	pixel get_bl(point x) const {
-		pixel result[2];
+	/*
+	 * Get a color value at a given position using bilinear interpolation between the
+	 * four nearest pixels.
+	 */
+	pixel get_bl(point x, int defined = 0) const {
+//		fprintf(stderr, "get_bl x=%f %f\n", (double) x[0], (double) x[1]);
 
-		get_bl(x, result);
+		pixel result;
 
-		return result[0];
+		assert (x[0] >= 0);
+		assert (x[1] >= 0);
+		assert (x[0] <= _dimy - 1);
+		assert (x[1] <= _dimx - 1);
+
+		int lx = (int) floor(x[1]);
+		int hx = (int) floor(x[1]) + 1;
+		int ly = (int) floor(x[0]);
+		int hy = (int) floor(x[0]) + 1;
+
+//		fprintf(stderr, "get_bl l=%d %d h=%d %d\n", ly, lx, hy, hx);
+
+		pixel neighbor[4];
+		ale_real factor[4];
+
+		neighbor[0] = get_pixel(ly, lx);
+		neighbor[1] = get_pixel(hy % _dimy, lx);
+		neighbor[2] = get_pixel(hy % _dimy, hx % _dimx);
+		neighbor[3] = get_pixel(ly, hx % _dimx);
+
+//		for (int d = 0; d < 4; d++)
+//			fprintf(stderr, "neighbor_%d=%f %f %f\n", d,
+//					(double) neighbor[d][0],
+//					(double) neighbor[d][1],
+//					(double) neighbor[d][2]);
+
+		factor[0] = (ale_real) (hx - x[1]) * (ale_real) (hy - x[0]);
+		factor[1] = (ale_real) (hx - x[1]) * (ale_real) (x[0] - ly);
+		factor[2] = (ale_real) (x[1] - lx) * (ale_real) (x[0] - ly);
+		factor[3] = (ale_real) (x[1] - lx) * (ale_real) (hy - x[0]);
+
+//		for (int d = 0; d < 4; d++)
+//			fprintf(stderr, "factor_%d=%f\n", d,
+//					(double) factor[d]);
+
+		/*
+		 * Use bilinear and/or geometric interpolation
+		 */
+
+		if (defined == 0) {
+			result = pixel(0, 0, 0);
+
+			for (int n = 0; n < 4; n++)
+				result += factor[n] * neighbor[n];
+		} else {
+#if 0
+			/*
+			 * Calculating the geometric mean may be expensive on
+			 * some platforms (e.g., those without floating-point
+			 * support.
+			 */
+
+			result = pixel(1, 1, 1);
+
+			for (int n = 0; n < 4; n++)
+				result *= ppow(neighbor[n], factor[n]);
+#else
+			/*
+			 * Taking the minimum value may be cheaper than
+			 * calculating a geometric mean.
+			 */
+
+			result = neighbor[0];
+
+			for (int n = 1; n < 4; n++) 
+			for (int k = 0; k < 3; k++) 
+				if (neighbor[n][k] < result[k])
+					result[k] = neighbor[n][k];
+#endif
+		}
+
+//		fprintf(stderr, "result=%f %f %f\n", 
+//				(double) result[0],
+//				(double) result[1],
+//				(double) result[2]);
+
+		return result;
 	}
 
-	void get_scaled_bl(point x, ale_pos f, pixel result[2]) const {
+	pixel get_scaled_bl(point x, ale_pos f, int defined = 0) const {
 		point scaled(
 				x[0]/f <= height() - 1
 			               ? (x[0]/f)
-				       : (height() - 1),
+				       : (ale_pos) (height() - 1),
 				x[1]/f <= width() - 1
 				       ? (x[1]/f)
-				       : (width() - 1));
+				       : (ale_pos) (width() - 1));
 
-		get_bl(scaled, result);
-	}
-
-	pixel get_scaled_bl(point x, ale_pos f) const {
-		pixel result[2];
-
-		get_scaled_bl(x, f, result);
-
-		return result[0];
+		return get_bl(scaled, defined);
 	}
 
 		
 	/*
 	 * Make a new image suitable for receiving scaled values.
 	 */
-	virtual image *scale_generator(int height, int width, int depth, char *name) const = 0;
+	virtual image *scale_generator(int height, int width, int depth, const char *name) const = 0;
 
 	/*
 	 * Generate an image of medians within a given radius
@@ -554,73 +563,103 @@ public:
 	 * At the edges, these values are normalized so that the sum of the
 	 * weights of contributing pixels is 1.
 	 */
-	image *scale_by_half(char *name) const {
+	class scale_by_half_threaded : public thread::decompose_domain {
+		image *is;
+		const image *iu;
+	protected:
+		void subdomain_algorithm(unsigned int thread,
+				int i_min, int i_max, int j_min, int j_max) {
+
+			ale_real _0625 = (ale_real) 0.0625;
+			ale_real _125  = (ale_real) 0.125;
+			ale_real _25   = (ale_real) 0.25;
+			ale_real _0    = (ale_real) 0;
+
+			unsigned int ui_min = (unsigned int) i_min;
+			unsigned int ui_max = (unsigned int) i_max;
+			unsigned int uj_min = (unsigned int) j_min;
+			unsigned int uj_max = (unsigned int) j_max;
+
+			for (unsigned int i = ui_min; i < ui_max; i++)
+			for (unsigned int j = uj_min; j < uj_max; j++) {
+				is->set_pixel(i, j, 
+
+				      ( ( ((i > 0 && j > 0) 
+					    ? iu->get_pixel(2 * i - 1, 2 * j - 1) * _0625
+					    : pixel(0, 0, 0))
+					+ ((i > 0)
+					    ? iu->get_pixel(2 * i - 1, 2 * j) * _125
+					    : pixel(0, 0, 0))
+					+ ((i > 0 && j < is->width() - 1)
+					    ? iu->get_pixel(2 * i - 1, 2 * j + 1) * _0625
+					    : pixel(0, 0, 0))
+					+ ((j > 0)
+					    ? iu->get_pixel(2 * i, 2 * j - 1) * _125
+					    : pixel(0, 0, 0))
+					+ iu->get_pixel(2 * i, 2 * j) * _25
+					+ ((j < is->width() - 1)
+					    ? iu->get_pixel(2 * i, 2 * j + 1) * _125
+					    : pixel(0, 0, 0))
+					+ ((i < is->height() - 1 && j > 0)
+					    ? iu->get_pixel(2 * i + 1, 2 * j - 1) * _0625
+					    : pixel(0, 0, 0))
+					+ ((i < is->height() - 1)
+					    ? iu->get_pixel(2 * i + 1, 2 * j) * _125
+					    : pixel(0, 0, 0))
+					+ ((i < is->height() && j < is->width() - 1)
+					    ? iu->get_pixel(2 * i + 1, 2 * j + 1) * _0625 
+					    : pixel(0, 0, 0)))
+
+				     /
+
+					( ((i > 0 && j > 0) 
+					    ? _0625
+					    : _0)
+					+ ((i > 0)
+					    ? _125
+					    : _0)
+					+ ((i > 0 && j < is->width() - 1)
+					    ? _0625
+					    : _0)
+					+ ((j > 0)
+					    ? _125
+					    : _0)
+					+ _25
+					+ ((j < is->width() - 1)
+					    ? _125
+					    : _0)
+					+ ((i < is->height() - 1 && j > 0)
+					    ? _0625
+					    : _0)
+					+ ((i < is->height() - 1)
+					    ? _125
+					    : _0)
+					+ ((i < is->height() && j < is->width() - 1)
+					    ? _0625
+					    : _0) ) ) );
+			}
+		}
+
+	public:
+		scale_by_half_threaded(image *_is, const image *_iu) 
+					: decompose_domain(0, _is->height(),
+					                   0, _is->width()) {
+			is = _is;	
+			iu = _iu;
+		}
+	};
+
+	image *scale_by_half(const char *name) const {
 		ale_pos f = 0.5;
 
 		image *is = scale_generator(
-			(int) floor(height() * f), 
-			(int) floor(width()  * f), depth(), name);
+			(int) floor(height() * (double) f), 
+			(int) floor(width()  * (double) f), depth(), name);
 
 		assert(is);
 
-		for (unsigned int i = 0; i < is->height(); i++)
-		for (unsigned int j = 0; j < is->width(); j++) 
-
-			is->set_pixel(i, j, 
-
-			      ( ( ((i > 0 && j > 0) 
-				    ? get_pixel(2 * i - 1, 2 * j - 1) * (ale_real) 0.0625
-				    : pixel(0, 0, 0))
-				+ ((i > 0)
-				    ? get_pixel(2 * i - 1, 2 * j) * 0.125
-				    : pixel(0, 0, 0))
-				+ ((i > 0 && j < is->width() - 1)
-				    ? get_pixel(2 * i - 1, 2 * j + 1) * 0.0625
-				    : pixel(0, 0, 0))
-				+ ((j > 0)
-				    ? get_pixel(2 * i, 2 * j - 1) * 0.125
-				    : pixel(0, 0, 0))
-				+ get_pixel(2 * i, 2 * j) * 0.25
-				+ ((j < is->width() - 1)
-				    ? get_pixel(2 * i, 2 * j + 1) * 0.125
-				    : pixel(0, 0, 0))
-				+ ((i < is->height() - 1 && j > 0)
-				    ? get_pixel(2 * i + 1, 2 * j - 1) * 0.0625
-				    : pixel(0, 0, 0))
-				+ ((i < is->height() - 1)
-				    ? get_pixel(2 * i + 1, 2 * j) * 0.125
-				    : pixel(0, 0, 0))
-				+ ((i < is->height() && j < is->width() - 1)
-				    ? get_pixel(2 * i + 1, 2 * j + 1) * 0.0625 
-				    : pixel(0, 0, 0)))
-
-			     /
-
-				( ((i > 0 && j > 0) 
-				    ? 0.0625
-				    : 0)
-				+ ((i > 0)
-				    ? 0.125
-				    : 0)
-				+ ((i > 0 && j < is->width() - 1)
-				    ? 0.0625
-				    : 0)
-				+ ((j > 0)
-				    ? 0.125
-				    : 0)
-				+ 0.25
-				+ ((j < is->width() - 1)
-				    ? 0.125
-				    : 0)
-				+ ((i < is->height() - 1 && j > 0)
-				    ? 0.0625
-				    : 0)
-				+ ((i < is->height() - 1)
-				    ? 0.125
-				    : 0)
-				+ ((i < is->height() && j < is->width() - 1)
-				    ? 0.0625
-				    : 0) ) ) );
+		scale_by_half_threaded sbht(is, this);
+		sbht.run();
 
 		is->_offset = point(_offset[0] * f, _offset[1] * f);
 
@@ -638,7 +677,7 @@ public:
 	 * Values are normalized so that the sum of the weights of contributing
 	 * pixels is 1.
 	 */
-	image *scale_by_half(const image *weights, char *name) const {
+	image *scale_by_half(const image *weights, const char *name) const {
 
 		if (weights == NULL)
 			return scale_by_half(name);
@@ -646,8 +685,8 @@ public:
 		ale_pos f = 0.5;
 
 		image *is = scale_generator(
-			(int) floor(height() * f), 
-			(int) floor(width()  * f), depth(), name);
+			(int) floor(height() * (double) f), 
+			(int) floor(width()  * (double) f), depth(), name);
 
 		assert(is);
 
@@ -749,47 +788,6 @@ public:
 		return is;
 	}
 
-	/*
-	 * Return an image scaled by some factor != 1.0, using bilinear
-	 * interpolation.
-	 */
-	image *scale(ale_pos f, char *name) const {
-
-		/*
-		 * We probably don't want to scale images by a factor of 1.0,
-		 * or by non-positive values.
-		 */
-		assert (f != 1.0 && f > 0);
-
-		if (f > 1.0) {
-			image *is = scale_generator(
-				(int) floor(height() * f), 
-				(int) floor(width()  * f), depth(), name);
-
-			assert(is);
-
-			unsigned int i, j, k;
-
-			for (i = 0; i < is->height(); i++)
-			for (j = 0; j < is->width(); j++) 
-			for (k = 0; k < is->depth(); k++)
-				is->set_pixel(i, j,
-					get_scaled_bl(point(i, j), f));
-
-			is->_offset = point(_offset[0] * f, _offset[1] * f);
-
-			return is;
-		} else if (f == 0.5) {
-			return scale_by_half(name);
-		} else {
-			image *is = scale(2*f, name);
-			image *result = is->scale(0.5, name);
-			delete is;
-			return result;
-		}
-
-	}
-
 	/*
 	 * Scale an image definition array by 1/2.
 	 *
@@ -802,65 +800,171 @@ public:
 	 *
 	 * Whereas scaling of image weight arrays is not generally obvious in
 	 * either purpose or method, ALE requires that image definition arrays
-	 * be scalable.
+	 * be scalable.  (Note that in the special case where weight is treated
+	 * as certainty, using a geometric mean is probably correct.)
 	 *
 	 * We currently use a geometric mean to implement scaling of
 	 * definition arrays.
 	 */
 
-	image *defined_scale_by_half(char *name) const {
+	class defined_scale_by_half_threaded : public thread::decompose_domain {
+		image *is;
+		const image *iu;
+	protected:
+		void subdomain_algorithm(unsigned int thread,
+				int i_min, int i_max, int j_min, int j_max) {
+
+#if 0
+			ale_real _0625 = (ale_real) 0.0625;
+			ale_real _125  = (ale_real) 0.125;
+			ale_real _25   = (ale_real) 0.25;
+#endif
+
+			int ui_min = (int) i_min;
+			int ui_max = (int) i_max;
+			int uj_min = (int) j_min;
+			int uj_max = (int) j_max;
+
+			for (int i = ui_min; i < ui_max; i++)
+			for (int j = uj_min; j < uj_max; j++) {
+
+#if 0
+
+				/*
+				 * Calculate a geometric mean; this approach
+				 * may be expensive on some platforms (e.g.,
+				 * those without floating-point support in
+				 * hardware).
+				 */
+
+				pixel value = pixel
+
+				      ( ( ((i > 0 && j > 0) 
+					    ? ppow(iu->get_pixel(2 * i - 1, 2 * j - 1), _0625) 
+					    : pixel(0, 0, 0))
+					* ((i > 0)
+					    ? ppow(iu->get_pixel(2 * i - 1, 2 * j), _125)
+					    : pixel(0, 0, 0))
+					* ((i > 0 && j < is->width() - 1)
+					    ? ppow(iu->get_pixel(2 * i - 1, 2 * j + 1), _0625)
+					    : pixel(0, 0, 0))
+					* ((j > 0)
+					    ? ppow(iu->get_pixel(2 * i, 2 * j - 1), _125)
+					    : pixel(0, 0, 0))
+					* ppow(iu->get_pixel(2 * i, 2 * j), _25) 
+					* ((j < is->width() - 1)
+					    ? ppow(iu->get_pixel(2 * i, 2 * j + 1), _125)
+					    : pixel(0, 0, 0))
+					* ((i < is->height() - 1 && j > 0)
+					    ? ppow(iu->get_pixel(2 * i + 1, 2 * j - 1), _0625)
+					    : pixel(0, 0, 0))
+					* ((i < is->height() - 1)
+					    ? ppow(iu->get_pixel(2 * i + 1, 2 * j), _125)
+					    : pixel(0, 0, 0))
+					* ((i < is->height() && j < is->width() - 1)
+					    ? ppow(iu->get_pixel(2 * i + 1, 2 * j + 1), _0625)
+					    : pixel(0, 0, 0))));
+#else
+
+				pixel value = iu->get_pixel(2 * i, 2 * j);
+
+				for (int ii = 2 * i - 1; ii <= 2 * i + 1; ii++)
+				for (int jj = 2 * j - 1; jj <= 2 * j + 1; jj++) {
+					if (ii < 0
+					 || jj < 0
+					 || ii > (int) iu->height() - 1
+					 || jj > (int) iu->height() - 1)
+						continue;
+
+					pixel value2 = iu->get_pixel(ii, jj);
+
+					for (int k = 0; k < 3; k++)
+						if (value2[k] < value[k]
+						 || !finite(value2[k]))   /* propagate non-finites */
+							value[k] = value2[k];
+				}
+
+#endif
+
+
+				for (int k = 0; k < 3; k++)
+					if (!finite(value[k]))
+						value[k] = 0;
+
+				is->set_pixel(i, j, value);
+			}
+		}
+
+	public:
+		defined_scale_by_half_threaded(image *_is, const image *_iu) 
+					: decompose_domain(0, _is->height(),
+					                   0, _is->width()) {
+			is = _is;	
+			iu = _iu;
+		}
+	};
+
+	image *defined_scale_by_half(const char *name) const {
 		ale_pos f = 0.5;
 
 		image *is = scale_generator(
-			(int) floor(height() * f), 
-			(int) floor(width()  * f), depth(), name);
+			(int) floor(height() * (double) f), 
+			(int) floor(width()  * (double) f), depth(), name);
 
 		assert(is);
 
-		for (unsigned int i = 0; i < is->height(); i++)
-		for (unsigned int j = 0; j < is->width(); j++) {
-
-			pixel value = pixel
-
-			      ( ( ((i > 0 && j > 0) 
-				    ? ppow(get_pixel(2 * i - 1, 2 * j - 1), 0.0625) 
-				    : pixel(0, 0, 0))
-				* ((i > 0)
-				    ? ppow(get_pixel(2 * i - 1, 2 * j), 0.125)
-				    : pixel(0, 0, 0))
-				* ((i > 0 && j < is->width() - 1)
-				    ? ppow(get_pixel(2 * i - 1, 2 * j + 1), 0.0625)
-				    : pixel(0, 0, 0))
-				* ((j > 0)
-				    ? ppow(get_pixel(2 * i, 2 * j - 1), 0.125)
-				    : pixel(0, 0, 0))
-				* ppow(get_pixel(2 * i, 2 * j), 0.25) 
-				* ((j < is->width() - 1)
-				    ? ppow(get_pixel(2 * i, 2 * j + 1), 0.125)
-				    : pixel(0, 0, 0))
-				* ((i < is->height() - 1 && j > 0)
-				    ? ppow(get_pixel(2 * i + 1, 2 * j - 1), 0.0625)
-				    : pixel(0, 0, 0))
-				* ((i < is->height() - 1)
-				    ? ppow(get_pixel(2 * i + 1, 2 * j), 0.125)
-				    : pixel(0, 0, 0))
-				* ((i < is->height() && j < is->width() - 1)
-				    ? ppow(get_pixel(2 * i + 1, 2 * j + 1), 0.0625)
-				    : pixel(0, 0, 0))));
-
-
-			for (int k = 0; k < 3; k++)
-				if (!finite(value[k]))
-					value[k] = 0;
-
-			is->set_pixel(i, j, value);
-		}
+		defined_scale_by_half_threaded dsbht(is, this);
+		dsbht.run();
 
 		is->_offset = point(_offset[0] * f, _offset[1] * f);
 
 		return is;
 	}
 
+	/*
+	 * Return an image scaled by some factor != 1.0, using bilinear
+	 * interpolation.
+	 */
+	image *scale(ale_pos f, const char *name, int defined = 0) const {
+
+		/*
+		 * We probably don't want to scale images by a factor of 1.0,
+		 * or by non-positive values.
+		 */
+		assert (f != 1.0 && f > 0);
+
+		if (f > 1.0) {
+			image *is = scale_generator(
+				(int) floor(height() * (double) f), 
+				(int) floor(width()  * (double) f), depth(), name);
+
+			assert(is);
+
+			unsigned int i, j, k;
+
+			for (i = 0; i < is->height(); i++)
+			for (j = 0; j < is->width(); j++) 
+			for (k = 0; k < is->depth(); k++)
+				is->set_pixel(i, j,
+					get_scaled_bl(point(i, j), f, defined));
+
+			is->_offset = point(_offset[0] * f, _offset[1] * f);
+
+			return is;
+		} else if (f == 0.5) {
+			if (defined == 0)
+				return scale_by_half(name);
+			else
+				return defined_scale_by_half(name);
+		} else {
+			image *is = scale(2*f, name, defined);
+			image *result = is->scale(0.5, name, defined);
+			delete is;
+			return result;
+		}
+
+	}
+
 	/*
 	 * Extend the image area to the top, bottom, left, and right,
 	 * initializing the new image areas with black pixels.  Negative values
@@ -871,7 +975,7 @@ public:
 	/*
 	 * Clone 
 	 */
-	image *clone(char *name) const {
+	image *clone(const char *name) const {
 		image *ic = scale_generator(
 			height(), width(), depth(), name);
 
@@ -961,7 +1065,7 @@ public:
 			for (k = 0; k < _depth; k++) 
 				if (finite(value[k])) {
 					accumulator += value[k];
-					divisor++;
+					divisor += 1;
 				}
 		}
 
