@@ -5,7 +5,7 @@
 
     The Anti-Lamenessing Engine is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
+    the Free Software Foundation; either version 3 of the License, or
     (at your option) any later version.
 
     The Anti-Lamenessing Engine is distributed in the hope that it will be useful,
@@ -41,7 +41,7 @@ private:
 
 public:
 	image_ale_real (unsigned int dimy, unsigned int dimx, unsigned int
-			depth, char *name = "anonymous", exposure *exp = NULL) 
+			depth, const char *name = "anonymous", exposure *exp = NULL) 
 			: image(dimy, dimx, depth, name, exp) {
 
 		_p = new spixel[dimx * dimy];
@@ -74,18 +74,18 @@ public:
 		return _p[y * _dimx + x];
 	}
 
-	ale_real &chan(unsigned int y, unsigned int x, unsigned int k) {
+	ale_sreal &chan(unsigned int y, unsigned int x, unsigned int k) {
 		return pix(y, x)[k];
 	}
 
-	const ale_real &chan(unsigned int y, unsigned int x, unsigned int k) const {
+	const ale_sreal &chan(unsigned int y, unsigned int x, unsigned int k) const {
 		return pix(y, x)[k];
 	}
 
 	/*
 	 * Make a new image suitable for receiving scaled values.
 	 */
-	virtual image *scale_generator(int height, int width, int depth, char *name) const {
+	virtual image *scale_generator(int height, int width, int depth, const char *name) const {
 		return new image_ale_real(height, width, depth, name, _exp);
 	}
 
