@@ -5,7 +5,7 @@
 
     The Anti-Lamenessing Engine is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
+    the Free Software Foundation; either version 3 of the License, or
     (at your option) any later version.
 
     The Anti-Lamenessing Engine is distributed in the hope that it will be useful,
@@ -51,7 +51,6 @@ class image_rw {
 	/*
 	 * Bit depth
 	 */
-	static unsigned int num_bits;
 	static unsigned int mcv;
 
 	/*
@@ -95,9 +94,20 @@ class image_rw {
 	static int latest_close_num;
 
 	/*
-	 * A cache of the data associated with the most recently closed image.
+	 * Maximum cache size, in megabytes (2^20 * bytes), for images not most
+	 * recently closed.
 	 */
-	static const image *latest_close;
+	static double cache_size_max;
+
+	/*
+	 * Actual cache size.
+	 */
+	static double cache_size;
+
+	/*
+	 * Number of cached files.
+	 */
+	static unsigned int cache_count;
 
 	/*
 	 * Private methods to init and shut down the file reader.
@@ -130,8 +140,11 @@ public:
 	/*
 	 * Read an image from a file
 	 */
-	static image *read_image(const char *filename, exposure *exp, char *name = "file", 
+	static image *read_image(const char *filename, exposure *exp, const char *name = "file", 
 			unsigned int bayer = IMAGE_BAYER_DEFAULT, int init_reference_gain = 0) {
+
+		static int warned = 0;
+
 		if (bayer == IMAGE_BAYER_DEFAULT)
 			bayer = bayer_default;
 
@@ -140,6 +153,16 @@ public:
 		}
 
 #ifdef USE_MAGICK
+
+                if (MaxRGB < 65535 && mcv == 65535 && !warned) {
+                        fprintf(stderr, "\n\n*** Warning: " MagickPackageName " has not been compiled with 16 bit support.\n");
+                        fprintf(stderr, "*** Reading input using 8 bits per channel.\n");
+                        fprintf(stderr, "*** \n"); 
+                        fprintf(stderr, "*** (To silence this warning, specify option --8bpc)\n\n\n");
+
+			warned = 1;
+                }
+
 		/*
 		 * Patterned after http://www.imagemagick.org/www/api.html
 		 * and http://www.imagemagick.org/www/smile.c
@@ -153,6 +176,8 @@ public:
 
 		unsigned int i, j;
 
+		ale_real black_level = exp->get_black_level();
+
 		GetExceptionInfo(&exception);
 		image_info = CloneImageInfo((ImageInfo *) NULL);
 
@@ -180,11 +205,15 @@ public:
 				exit(1);
 
 			for (j = 0; j < mi->columns; j++) {
-				pixel input( (ale_real) p->red / MaxRGB,
-					     (ale_real) p->green / MaxRGB,
-					     (ale_real) p->blue / MaxRGB );
 
-				im->set_pixel(i, j, exp->linearize(input));
+				pixel input ( ale_real_from_int(p->red, MaxRGB),
+					      ale_real_from_int(p->green, MaxRGB),
+					      ale_real_from_int(p->blue, MaxRGB) );
+
+				pixel linear_input = (exp->linearize(input) - exp->get_multiplier() * black_level)
+					           / (1 - black_level);
+				
+				im->set_pixel(i, j, linear_input);
 
 				p++;
 			}
@@ -240,9 +269,7 @@ public:
 		for (unsigned int i = 0; i < file_count; i++)
 			bayer_specific[i] = IMAGE_BAYER_DEFAULT;
 
-		fprintf(stderr, "Output file will be '%s'.\n", 
-				output_filename);
-
+		ui::get()->identify_output(output_filename);
 	}
 
 	static void ppm_plain() {
@@ -273,15 +300,17 @@ public:
 	}
 
 	static void depth16() {
-		num_bits = 16;
 		mcv = 65535;
 	}
 
 	static void depth8() {
-		num_bits = 8;
 		mcv = 255;
 	}
 
+	static void set_cache(double size) {
+		cache_size_max = size;
+	}
+
 	static void destroy() {
 		assert (file_count > 0);
 		destroy_image();
@@ -311,6 +340,8 @@ public:
 	 * Write an image to a file
 	 */
 	static void write_image(const char *filename, const image *im, exposure *exp = output_exposure, int rezero = 0, int exp_scale_override = 0) {
+		static int warned = 0;
+
 		/*
 		 * Handle ALE-specific magical filenames.
 		 */
@@ -319,8 +350,8 @@ public:
 			fprintf(stderr, "Image dump: ");
 			for (unsigned int i = 0; i < im->height(); i++)
 			for (unsigned int j = 0; j < im->width(); j++) {
-				pixel p = im->pix(i, j);
-				fprintf(stderr, "(%d, %d): [%f %f %f] ", i, j, p[0], p[1], p[2]);
+				pixel p = im->get_pixel(i, j);
+				fprintf(stderr, "(%d, %d): [%f %f %f] ", i, j, (double) p[0], (double) p[1], (double) p[2]);
 			}
 			fprintf(stderr, "\n");
 
@@ -362,10 +393,13 @@ public:
 		else
 			mi->depth = 16;
 
-		if (MaxRGB < 65535 && mcv == 65535) {
-			fprintf(stderr, "\n\n*** Warning: 16 bit-per-channel file output was specified,\n");
-			fprintf(stderr, "*** but ImageMagick has not been compiled with support for this.\n");
+		if (MaxRGB < 65535 && mcv == 65535 && !warned) {
+			fprintf(stderr, "\n\n*** Warning: " MagickPackageName " has not been compiled with 16 bit support.\n");
 			fprintf(stderr, "*** Writing output using 8 bits per channel.\n");
+                        fprintf(stderr, "*** \n"); 
+                        fprintf(stderr, "*** (To silence this warning, specify option --8bpc)\n\n\n");
+
+			warned = 1;
 		}
 
 		/*
@@ -386,7 +420,7 @@ public:
 		 * Automatic exposure adjustment (don't blow out highlights)
 		 */
 		ale_real maxval = 1;
-		ale_real minval = (rezero ? im->minval() : 0);
+		ale_real minval = (rezero ? im->minval() : (ale_real) 0);
 		if (minval > 0)
 			minval = 0;
 		pixel minval_pixel(minval, minval, minval);
@@ -449,9 +483,9 @@ public:
 
 				unlinearized = unlinearized.clamp();
 
-				p->red = (Quantum) round(unlinearized[0] * MaxRGB);
-				p->green = (Quantum) round(unlinearized[1] * MaxRGB);
-				p->blue = (Quantum) round(unlinearized[2] * MaxRGB);
+				p->red =   (Quantum) ale_real_to_int(unlinearized[0], MaxRGB);
+				p->green = (Quantum) ale_real_to_int(unlinearized[1], MaxRGB);
+				p->blue =  (Quantum) ale_real_to_int(unlinearized[2], MaxRGB);
 				p++;
 			}
 
@@ -536,17 +570,21 @@ public:
 		assert (n <  file_count);
 		assert (!files_open[n]);
 
+		files_open[n] = 1;
+
 		if (latest_close_num >= 0 && n == (unsigned int) latest_close_num) {
-			images[n] = latest_close;
 			latest_close_num = -1;
-			files_open[n] = 1;
-		} else {
+			return images[n];
+		} 
+		
+		if (n < cache_count)
+			return images[n];
 
-			image *i = read_image(filenames[n], input_exposure[n], "file", bayer(n), (n == 0));
+		ui::get()->loading_file();
+		image *i = read_image(filenames[n], input_exposure[n], "file", bayer(n), (n == 0));
+
+		images[n] = i;
 
-			images[n] = i;
-			files_open[n] = 1;
-		}
 		return images[n];
 	}
 
@@ -560,7 +598,7 @@ public:
 		return images[n];
 	}
 
-	static image *copy(unsigned int n, char *name) {
+	static image *copy(unsigned int n, const char *name) {
 		assert (n <  file_count);
 
 		if (files_open[n])
@@ -575,12 +613,28 @@ public:
 		assert (image <  file_count);
 		assert (files_open[image]);
 
-		if (latest_close_num >= 0)
-			delete latest_close;
-		latest_close = images[image];
-		latest_close_num = image;
-
 		files_open[image] = 0;
+
+		if (image < cache_count)
+			return;
+
+		if (image == cache_count) {
+			double image_size = ((double) images[image]->storage_size()) / pow(2, 20);
+
+			if (image_size + cache_size < cache_size_max) {
+				cache_size += image_size;
+				cache_count++;
+				ui::get()->cache(cache_size, cache_size_max);
+				return;
+			} else {
+				ui::get()->cache_status(0);
+			}
+		}
+
+		if (latest_close_num >= 0)
+			delete images[latest_close_num];
+
+		latest_close_num = image;
 	}
 
 	static void close_all() {
