@@ -5,7 +5,7 @@
 
     The Anti-Lamenessing Engine is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
+    the Free Software Foundation; either version 3 of the License, or
     (at your option) any later version.
 
     The Anti-Lamenessing Engine is distributed in the hope that it will be useful,
@@ -35,7 +35,7 @@
 
 struct extended_t {
 	int is_extended;
-	int black_level;
+	ale_real black_level;
 	ale_real aperture; /* 1 == f/1.0, 1.4 == f/1.4, etc. */
 	ale_real shutter;  /* 1 == 1 sec, 0.5 == 1/2 sec, etc. */
 	ale_real gain;     /* 1 == ISO 100, 2 == ISO 200, etc. */
@@ -43,19 +43,17 @@ struct extended_t {
 	extended_t() {
 		is_extended = 0;
 		black_level = 0;
-		aperture = 1;
-		shutter = 1;
-		gain = 1;
+		aperture = 0;
+		shutter = 0;
+		gain = 0;
 	}
 };
 
 static inline void error_ppm(const char *filename) {
 	fprintf(stderr, 
 		"\n\n*** '%s' doesn't look like a PPM file.\n"
-		"\n*** To handle other file types, compile ALE with\n"
-		"*** ImageMagick support ('make IMAGEMAGICK=1').\n"
-		"*** (To do this, you must have a source distribution\n"
-		"*** of ALE, and you must have ImageMagick installed.)\n\n", 
+		"\n*** (To handle other file types, use a version of ALE with\n"
+		"*** ImageMagick support enabled.)\n\n",
 		filename);
 	exit(1);
 }
@@ -208,6 +206,12 @@ static inline image *read_ppm(const char *filename, exposure *e, unsigned int ba
 	assert(n == 1);
 	assert(mcv <= 65535 || m2 == '3');
 
+	if (extended.black_level == 0) {
+		extended.black_level = e->get_black_level();
+	} else {
+		extended.black_level /= mcv;
+	}
+
 	if (n != 1 || (mcv > 65535 && m2 == '6'))
 		error_ppm(filename);
 
@@ -269,24 +273,40 @@ static inline image *read_ppm(const char *filename, exposure *e, unsigned int ba
 					error_ppm(filename);
 			}
 
-			p[k] = (ale_real) (ival - (extended.is_extended ? extended.black_level : 0)) 
-			     / (ale_real) (mcv  - (extended.is_extended ? extended.black_level : 0));
+			p[k] = ale_real_from_int(ival, mcv);
+
 		}
 
-		im->set_pixel(i, j, e->linearize(p));
+		pixel p_linear = (e->linearize(p) - e->get_multiplier() * extended.black_level)
+			       / (1 - extended.black_level);
+
+		im->set_pixel(i, j, p_linear);
 	}
 
 	/* Handle exposure and gain */
 
 	if (extended.is_extended) {
-		double combined_gain = (1 / pow(extended.aperture, 2))
-			             * extended.shutter
-			             * extended.gain;
-		if (init_reference_gain)
-			exposure::set_gain_reference(combined_gain);
-		else
-			e->set_gain_multiplier(exposure::get_gain_reference()
-					     / combined_gain);
+		if (extended.aperture != 0
+		 || extended.shutter != 0
+		 || extended.gain != 0) {
+
+			if (extended.aperture == 0)
+				extended.aperture = 1;
+			if (extended.shutter == 0)
+				extended.shutter = 1;
+			if (extended.gain == 0)
+				extended.gain = 1;
+
+			ale_real combined_gain = (1 / pow(extended.aperture, 2))
+					     * extended.shutter
+					     * extended.gain;
+			
+			if (init_reference_gain)
+				exposure::set_gain_reference(combined_gain);
+			else
+				e->set_gain_multiplier(exposure::get_gain_reference()
+						     / combined_gain);
+		}
 	}
 
 	/* Done */
@@ -334,7 +354,7 @@ static inline void write_ppm(const char *filename, const image *im, exposure *e,
 	/* Automatic exposure adjustment information */
 
 	ale_real maxval = 1;
-	ale_real minval = (rezero ? im->minval() : 0);
+	ale_real minval = (rezero ? im->minval() : (ale_real) 0);
 	if (minval > 0)
 		minval = 0;
 	pixel minval_pixel(minval, minval, minval);
@@ -384,11 +404,9 @@ static inline void write_ppm(const char *filename, const image *im, exposure *e,
 			              / (maxval - minval);
 		pixel unlinearized = (e->unlinearize(exposure_adjust)).clamp();
 
-		pixel rescaled = unlinearized * (ale_real) mcv;
-
 		for (k = 0; k < im->depth();  k++) {
 
-			uint16_t output_value = (uint16_t) round(rescaled[k]);
+			uint16_t output_value = (uint16_t) ale_real_to_int(unlinearized[k], mcv);
 
 			if (plain) {
 				fprintf(f, "%d ", output_value);
