@@ -5,7 +5,7 @@
 
     The Anti-Lamenessing Engine is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
+    the Free Software Foundation; either version 3 of the License, or
     (at your option) any later version.
 
     The Anti-Lamenessing Engine is distributed in the hope that it will be useful,
@@ -38,7 +38,7 @@ public:
 		this->psf_match_args = psf_match_args;
 	}
 
-        void _ip_frame(ale_accum *diff, unsigned long *count, int m) {
+        void _ip_frame(ale_accum *diff, unsigned int *count, int m) {
 
 		/*
 		 * Get alignment information for frame m.
@@ -90,10 +90,10 @@ public:
 			 * response function.
 			 */
 
-                        for (int ii = (int) floor(q[0] - d[0] + lresponse->min_i());
-                                ii <= ceil(q[0] + d[0] + lresponse->max_i()); ii++)
-                        for (int jj = (int) floor(q[1] - d[1] + lresponse->min_j());
-                                jj <= ceil(q[1] + d[1] + lresponse->max_j()); jj++) {
+                        for (int ii = (int) floor(q[0] - d[0] + (ale_pos) lresponse->min_i());
+                                ii <= ceil(q[0] + d[0] + (ale_pos) lresponse->max_i()); ii++)
+                        for (int jj = (int) floor(q[1] - d[1] + (ale_pos) lresponse->min_j());
+                                jj <= ceil(q[1] + d[1] + (ale_pos) lresponse->max_j()); jj++) {
 
                                 ale_pos top = q[0] - d[0];
                                 ale_pos bot = q[0] + d[0];
@@ -201,10 +201,10 @@ public:
 		 * of differences.
 		 */
 
-		ale_pos margin_i1 = lresponse->min_i() + (nlresponse ? nlresponse->min_i() : 0);
-		ale_pos margin_i2 = lresponse->max_i() + (nlresponse ? nlresponse->max_i() : 0);
-		ale_pos margin_j1 = lresponse->min_j() + (nlresponse ? nlresponse->min_j() : 0);
-		ale_pos margin_j2 = lresponse->max_j() + (nlresponse ? nlresponse->max_j() : 0);
+		ale_real margin_i1 = lresponse->min_i() + (nlresponse ? nlresponse->min_i() : ale_real_0);
+		ale_real margin_i2 = lresponse->max_i() + (nlresponse ? nlresponse->max_i() : ale_real_0);
+		ale_real margin_j1 = lresponse->min_j() + (nlresponse ? nlresponse->min_j() : ale_real_0);
+		ale_real margin_j2 = lresponse->max_j() + (nlresponse ? nlresponse->max_j() : ale_real_0);
 
 		for (unsigned int i = 0; i < simulated->height(); i++)
 		for (unsigned int j = 0; j < simulated->width();  j++) {
@@ -286,7 +286,7 @@ public:
 		 */
 
 		ale_accum diff = 0;
-		unsigned long channel_count = 0;
+		unsigned int channel_count = 0;
 
 		approximation = image_rw::copy(image_rw::count() - 1, "PSF_CALIBRATE reference");
 
@@ -313,7 +313,7 @@ public:
 			_ip_frame(&diff, &channel_count, m);
 		}
 
-		diff = pow(diff / channel_count, 0.5);
+		diff = pow(diff / (ale_accum) channel_count, 0.5);
 
 		fprintf(stderr, "\n\nPSF Error:: %e\n\n", (double) diff);
 
