@@ -1,11 +1,11 @@
-// Copyright 2002, 2003 David Hilvert <dhilvert@auricle.dyndns.org>,
-//                                    <dhilvert@ugcs.caltech.edu>
+// Copyright 2002, 2003, 2007 David Hilvert <dhilvert@auricle.dyndns.org>,
+//                                          <dhilvert@ugcs.caltech.edu>
 
 /*  This file is part of the Anti-Lamenessing Engine.
 
     The Anti-Lamenessing Engine is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
+    the Free Software Foundation; either version 3 of the License, or
     (at your option) any later version.
 
     The Anti-Lamenessing Engine is distributed in the hope that it will be useful,
@@ -293,7 +293,7 @@ static inline transformation tload_first(struct tload_t *t, int is_p,
 					int count, i;
 					point x[4];
 
-					count = sscanf(line, "P %lf%lf%lf%lf%lf%lf%lf%lf%lf%lf", &width, &height,
+					count = sscanf(line + 1, " %lf%lf%lf%lf%lf%lf%lf%lf%lf%lf", &width, &height,
 							&values[0], &values[1], &values[2], &values[3],
 							&values[4], &values[5], &values[6], &values[7]);
 				
@@ -308,8 +308,8 @@ static inline transformation tload_first(struct tload_t *t, int is_p,
 
 					for (i = 0; i < count - 2; i++) {
 						ale_pos factor = (i % 2)
-							? (result.scaled_width() / width)
-							: (result.scaled_height() / height);
+							? ((double) result.scaled_width() / width)
+							: ((double) result.scaled_height() / height);
 							
 						x[i / 2][i % 2] *= factor;
 					}
@@ -329,7 +329,7 @@ static inline transformation tload_first(struct tload_t *t, int is_p,
 					int count, i;
 					ale_pos eu[3];
 
-					count = sscanf(line, "E %lf%lf%lf%lf%lf",
+					count = sscanf(line + 1, " %lf%lf%lf%lf%lf",
 							&width, &height,
 							&values[0], &values[1], &values[2]);
 
@@ -343,8 +343,8 @@ static inline transformation tload_first(struct tload_t *t, int is_p,
 
 					for (i = 0; (i < count - 2) && (i < 2); i++) {
 						ale_pos factor = (i % 2)
-							? (result.scaled_width() / width)
-							: (result.scaled_height() / height);
+							? ((double) result.scaled_width() / width)
+							: ((double) result.scaled_height() / height);
 							
 						eu[i] *= factor;
 					}
@@ -381,10 +381,14 @@ static inline transformation tload_first(struct tload_t *t, int is_p,
  * 	DEFAULT_TRANSFORM is the default transformation result.
  *
  *	IS_DEFAULT is used to signal a non-default transformation result.
+ *
+ *	IS_PRIMARY is used to differentiate primary and non-primary 
+ *	transformations
  */
 
 static inline transformation tload_next(struct tload_t *t, int is_p, 
-		transformation default_transform, int *is_default) {
+		transformation default_transform, int *is_default, 
+		int is_primary) {
 
 	transformation result = default_transform;
 
@@ -395,6 +399,25 @@ static inline transformation tload_next(struct tload_t *t, int is_p,
 	 */
 
 	while (t && !feof(t->file)) {
+
+		char c = fgetc(t->file);
+		if (!feof(t->file) && c != EOF)
+			ungetc(c, t->file);
+
+		if (feof(t->file)
+		 || (!is_primary
+		  && c != EOF
+		  && (c == 'E'
+		   || c == 'e'
+		   || c == 'P'
+		   || c == 'p'
+		   || c == 'D'
+		   || c == 'd'
+		   || c == 'B'
+		   || c == 'b'))) {
+			return result;
+		}
+
 		char line[1024];
 
 		fgets(line, 1024, t->file);
@@ -470,6 +493,10 @@ static inline transformation tload_next(struct tload_t *t, int is_p,
 					result.bd_set(bdc, parameters);
 				}
 				break;
+			case 'Q':
+			case 'q':
+				if (is_primary)
+					break;
 			case 'P':
 			case 'p':
 				/* Projective transformation data */
@@ -486,7 +513,7 @@ static inline transformation tload_next(struct tload_t *t, int is_p,
 					int count, i;
 					point x[4];
 
-					count = sscanf(line, "P %lf%lf%lf%lf%lf%lf%lf%lf%lf%lf", &width, &height,
+					count = sscanf(line + 1, " %lf%lf%lf%lf%lf%lf%lf%lf%lf%lf", &width, &height,
 							&values[0], &values[1], &values[2], &values[3],
 							&values[4], &values[5], &values[6], &values[7]);
 				
@@ -501,8 +528,8 @@ static inline transformation tload_next(struct tload_t *t, int is_p,
 
 					for (i = 0; i < count - 2; i++) {
 						ale_pos factor = (i % 2)
-							? (result.scaled_width() / width)
-							: (result.scaled_height() / height);
+							? ((double) result.scaled_width() / width)
+							: ((double) result.scaled_height() / height);
 							
 						x[i / 2][i % 2] *= factor;
 					}
@@ -525,6 +552,10 @@ static inline transformation tload_next(struct tload_t *t, int is_p,
 					return result;
 				}
 				break;
+			case 'F':
+			case 'f':
+				if (is_primary)
+					break;
 			case 'E':
 			case 'e':
 				/* Euclidean transformation data */
@@ -535,7 +566,7 @@ static inline transformation tload_next(struct tload_t *t, int is_p,
 					int count, i;
 					ale_pos eu[3];
 
-					count = sscanf(line, "E %lf%lf%lf%lf%lf",
+					count = sscanf(line + 1, " %lf%lf%lf%lf%lf",
 							&width, &height,
 							&values[0], &values[1], &values[2]);
 
@@ -556,8 +587,8 @@ static inline transformation tload_next(struct tload_t *t, int is_p,
 
 					for (i = 0; (i < count - 2) && (i < 2); i++) {
 						ale_pos factor = (i % 2)
-							? (result.scaled_width() / width)
-							: (result.scaled_height() / height);
+							? ((double) result.scaled_width() / width)
+							: ((double) result.scaled_height() / height);
 							
 						eu[i] *= factor;
 					}
@@ -652,7 +683,7 @@ static inline void tsave_first(struct tsave_t *t, transformation offset, int is_
 
 	fprintf(t->file, "# Comment: Target output file is %s\n", t->target);
 	fprintf(t->file, "# Comment: Original frame is %s\n", t->orig);
-	fprintf(t->file, "# Comment: Avg magnitude [r=%f g=%f b=%f]\n", t->orig_apm[0], t->orig_apm[1], t->orig_apm[2]);
+	fprintf(t->file, "# Comment: Avg magnitude [r=%f g=%f b=%f]\n", (double) t->orig_apm[0], (double) t->orig_apm[1], (double) t->orig_apm[2]);
 
 	if (tfile_output_version < 3) {
 		fclose(t->file);
@@ -700,16 +731,19 @@ static inline void tsave_first(struct tsave_t *t, transformation offset, int is_
  *
  * 	IS_PROJECTIVE indicates whether to write a projective transformation.
  *
+ * 	IS_PRIMARY indicates whether to write a primary transformation
+ *
  */
 
-static inline void tsave_next(struct tsave_t *t, transformation offset, int is_projective) {
+static inline void tsave_next(struct tsave_t *t, transformation offset, int is_projective,
+		int is_primary) {
 
 	if (t == NULL)
 		return;
 
 	t->file = fopen(t->filename, "a");
 	
-	if (offset.bd_count() > 0) {
+	if (is_primary && offset.bd_count() > 0) {
 		assert (tfile_output_version >= 3);
 		unsigned int i;
 
@@ -723,13 +757,13 @@ static inline void tsave_next(struct tsave_t *t, transformation offset, int is_p
 	if (is_projective) {
 		int i, j;
 
-		fprintf(t->file, "P ");
+		fprintf(t->file, is_primary ? "P " : "Q ");
 		fprintf(t->file, "%f %f ", (double) offset.scaled_width(), (double) offset.scaled_height());
 		for (i = 0; i < 4; i++)
 			for (j = 1; j >= 0; j--)
 				fprintf(t->file, "%f ", (double) offset.gpt_get(i, j));
 	} else {
-		fprintf(t->file, "E ");
+		fprintf(t->file, is_primary ? "E " : "F ");
 		fprintf(t->file, "%f %f ", (double) offset.scaled_width(), (double) offset.scaled_height());
 		fprintf(t->file, "%f ",    (double) offset.eu_get(1));
 		fprintf(t->file, "%f ",    (double) offset.eu_get(0));
@@ -798,7 +832,7 @@ static inline void tsave_trm(struct tsave_t *t, ale_real r, ale_real g, ale_real
        if (t != NULL) {
                t->file = fopen(t->filename, "a");
 
-               fprintf(t->file, "# Comment: Exposure [r=%f g=%f b=%f]\n", r, g, b);
+               fprintf(t->file, "# Comment: Exposure [r=%f g=%f b=%f]\n", (double) r, (double) g, (double) b);
 
                fclose(t->file);
        }
@@ -813,7 +847,7 @@ static inline void tsave_apm(struct tsave_t *t, ale_real r, ale_real g, ale_real
        if (t != NULL) {
                t->file = fopen(t->filename, "a");
 
-               fprintf(t->file, "# Comment: Avg magnitude [r=%f g=%f b=%f]\n", r, g, b);
+               fprintf(t->file, "# Comment: Avg magnitude [r=%f g=%f b=%f]\n", (double) r, (double) g, (double) b);
 
                fclose(t->file);
        }
