@@ -5,7 +5,7 @@
 
     The Anti-Lamenessing Engine is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
+    the Free Software Foundation; either version 3 of the License, or
     (at your option) any later version.
 
     The Anti-Lamenessing Engine is distributed in the hope that it will be useful,
@@ -87,7 +87,6 @@ public:
 			"Help options:\n"
 			HEADER_SPACE
 			"--hu              Usage (this message).\n"
-			"--hq              Default settings.\n"
 			"--hf              Image files.\n"
 			"--he              Exclusion regions.\n"
 			"--ha              Alignment (not exposure-related).\n"
@@ -114,38 +113,7 @@ public:
 			BETWEEN_SECTIONS
 			"Default settings:\n"
 			HEADER_SPACE
-			"--q0              Low quality, high speed.\n"
-			"--q1              Moderate quality and speed.\n"
-			"--q2              High quality, low speed. [default]\n"
-			"--qn              Low noise, moderate speed.\n"
-			BETWEEN_SECTIONS
-			"q0 defaults:\n"
-			HEADER_SPACE
-			"   --dchain fine:triangle:2,triangle:2\n"
-			"   --achain triangle:2\n"
-			"   --ips 0\n"
-			"   --3d-chain fine:triangle:2,triangle:2\n"
-			BETWEEN_SECTIONS
-			"q1 defaults:\n"
-			HEADER_SPACE
-			"   --dchain fine:triangle:2,fine:gauss:0.75,triangle:2\n"
-			"   --achain triangle:2\n"
-			"   --ips 0\n"
-			"   --3d-chain fine:triangle:2,fine:gauss:0.75,triangle:2\n"
-			BETWEEN_SECTIONS
-			"q2 defaults:\n"
-			HEADER_SPACE
-			"   --dchain triangle:2\n"
-			"   --achain triangle:2\n"
-			"   --ips 4\n"
-			"   --3d-chain fine:triangle:2,fine:gauss:0.75,triangle:2\n"
-			BETWEEN_SECTIONS
-			"Low noise defaults:\n"
-			HEADER_SPACE
-			"   --dchain sinc*lanc:6\n"
-			"   --achain sinc*lanc:6\n"
-			"   --ips 0\n"
-			"   --3d-chain sinc*lanc:6\n"
+			"--q* options are no longer recognized.\n"
 			"\n"
 		       );
 	}
@@ -153,11 +121,15 @@ public:
 	void file() {
 		banner("File");
 		fprintf(help_stream,
+			BETWEEN_SECTIONS
+			"Cache options:\n"
+			HEADER_SPACE
+			"--cache <size>    Cache up to <size> megabytes of image data.   (256 is default)\n"
 			BETWEEN_SECTIONS
 			"Bit depth options:\n"
 			HEADER_SPACE
-			"--8bpc            Write 8 bit per channel output [default]\n"
-			"--16bpc           Write 16 bit per channel output\n"
+			"--8bpc            Write 8 bit per channel output\n"
+			"--16bpc           Write 16 bit per channel output [default]\n"
 			BETWEEN_SECTIONS
 			"Output format options:\n"
 			HEADER_SPACE
@@ -172,8 +144,8 @@ public:
 			BETWEEN_SECTIONS
 			"Incremental output:\n"
 			HEADER_SPACE
-			"--inc             Produce incremental output.  [default]\n"
-			"--no-inc          Don't produce incremental output.\n"
+			"--inc             Produce rough incremental output.\n"
+			"--no-inc          Don't produce any incremental output.  [default]\n"
 
 			BETWEEN_SECTIONS
 			"Undefined values:\n"
@@ -214,13 +186,6 @@ public:
 			"--trans-load <x>  Load initial transformation settings from file <x>\n"
 			"--trans-save <x>  Save final transformation data in file <x>\n"
 			BETWEEN_SECTIONS
-			"Monte Carlo alignment:\n"
-			HEADER_SPACE
-			"--mc <x>          Statically sample ~x%% of available pixels       (0 < x < 100)\n"
-                        "--no-mc           Statically sample all pixels.\n"
-			"--mcd             Dynamically sample with robustness checks.          [default]\n"
-			"--mcd-removal <x> Check robustness against removing x pixels.  (100 is default)\n"
-			BETWEEN_SECTIONS
 			"Tunable parameters:\n"
 			HEADER_SPACE
 			"--metric=x        Set the alignment error metric exponent.       (2 is default)\n"
@@ -230,9 +195,10 @@ public:
 			"--perturb-lower=x Perturbation lower bound pixels/arclength   (.125 is default)\n"
 			"                     ('x%%' uses a fraction of the smallest image dimension.)\n"
 			"--rot-upper=x     Rotation-specific upper bound in degrees    (32.0 is default)\n"
-			"--bda-mult=x      Barrel distortion adjustment multiplier   (0.0001 is default)\n"
-			"--bda-rate=x      Barrel distortion rate of change maximum  (0.0004 is default)\n"   
-			"--lod-max=x       LOD scale factor is max(1, (2^floor(x))/perturb)  (1 is def.)\n"
+			"--bda-mult=x      Barrel distortion adjustment multiplier      (2.0 is default)\n"
+			"--bda-rate=x      Barrel distortion rate of change maximum     (8.0 is default)\n"   
+			"--lod-max=x       LOD scale factor is max(1, (2^floor(x))/perturb)  (-4 is def)\n"
+			"--mc <x>          Statically sample ~x%% of available pixels     (30 is default)\n"
 			BETWEEN_SECTIONS
 			"Certainty-weighted alignment:\n"
 			HEADER_SPACE
@@ -275,13 +241,19 @@ public:
 			"Global searching:\n"
 			HEADER_SPACE
 			"--gs <type>       Set global search to <type>, one of:\n"
-			"                     local     Local alignment only [default]\n"
+			"                     local     Local alignment only\n"
 			"                     inner     Alignment reference image inner region\n"
 			"                     outer     Alignment reference image outer region\n"
 			"                     all       Union of inner and outer\n"
 			"                     central   inner if below threshold or better; else, outer.\n"
+			"                     defaults  'all' if default, 'local' otherwise.  [default]\n"
 			"                     points    Align by control points.  Ignores gs-mo.\n"
-			"--gs-mo <x>       Set <x> pixel min. overlap for global search. (16 is default)\n"
+			"--gs-mo <x>       Set <x> pixel min. overlap for global search.   (67%% default)\n"
+			"                     ('x%%' uses a fraction of the total pixel count.)\n"
+			BETWEEN_SECTIONS
+			"Multi-alignment:\n"
+			HEADER_SPACE
+			"--md <n>          Use no more than <n>-fold decomposition.       (3 is default)\n"
 			"\n"
 		       );
 	}
@@ -305,9 +277,15 @@ public:
 			"--scale=x         Scale images by the factor x, where x > 0.     (1 is default)\n"
 			"--threshold=x     Min. match threshold; a perfect match is 100.  (0 is default)\n"
 			BETWEEN_SECTIONS
-			"Irani-Peleg iterative solver (see --hq for --ips defaults):\n"
+			"Irani-Peleg iterative solver:\n"
 			HEADER_SPACE
-			"--ips <i>         Run <i> iterations.  (see also --hx, --hl and --hd)\n"
+#define FIXED16 4
+#if ALE_COLORS == FIXED16
+			"--ips <i>         Run <i> iterations.                            (0 is default)\n"
+#else
+			"--ips <i>         Run <i> iterations.                            (1 is default)\n"
+#endif
+#undef FIXED16
 			"--ip-mean         Use the mean correction [default]\n"
 			"--ip-median       Use the median correction\n"
 			"--ip-wl <x>       Use weight limit <x>\n"
@@ -348,6 +326,7 @@ public:
 			"                     Available point-spread functions:\n"
 			"                        box=<diameter>\n"
 			"                        circle=<diameter>\n"
+			"                        gauss=<diameter of one std. deviation>\n"
 			"                        stdin\n"
 			"                        stdin_vg\n"
 			"                        <p>+<p> (summation)\n"
@@ -356,7 +335,7 @@ public:
 			"                     Default lpsf is either 'box=1.0' or device-specific.\n"
 			"                     Default nlpsf is either disabled or device-specific.\n"
 			BETWEEN_SECTIONS
-			"Incremental rendering chains:\n"
+			"Rendering chains:\n"
 			HEADER_SPACE
 			"--wt <w>          Set weight threshold for defined pixels (default is 0.0001).\n"
 			"--dchain <g>      Use chain <g> to render the default output.\n"
@@ -371,6 +350,7 @@ public:
 			"                     last:nex:sinc*lanc:8         Useful for video stabilization\n"
 			"                     fine:box:1,triangle:2        Multi-resolution rendering\n"
 			"                     fine:sinc*lanc:8,sinc*lanc:8 Multi-resolution rendering\n"
+			"                     auto:box:1,fine:box:1,box:1  Range-resolution rendering.\n"
 			"                  More precisely, chains <g> are one of:\n"
 			"                     <g1>,<g2>      Chain <g1> where defined; <g2> elsewhere.\n"
 			"                     <i>            Use rendering invariant <i>.\n"
@@ -388,6 +368,7 @@ public:
 			"                     nex:<s>        Use SSF <s>; don't honor exclusion regions.\n"
 			"                     <s>            Same as ex:<s>\n"
 			"                  Scaled sampling filters (SSF) <s> are:\n"
+			"                     auto:<f>       filter <f> from chain suffix resolutions.\n"
 			"                     fine:<f>       filter <f> at output image resolution.\n"
 			"                     coarse:<f>     filter <f> at resolution MIN(in, out).\n"
 			"                     <f>            Same as coarse:<f>.\n"
@@ -400,8 +381,9 @@ public:
 			"                     zero           Zero function.\n"
 			"                     <f>*<f>        Pointwise multiplication (windowing).\n"
 			"                  Defaults:\n"
-			"                     dchain         (see --hq)\n"
-			"                     achain         (see --hq)\n"
+			"                     dchain         auto:triangle:2,fine:box:1,triangle:2\n"
+			"                     3d-chain       fine:triangle:2,fine:gauss:0.75,triangle:2\n"
+			"                     achain         triangle:2\n"
 			"                     afilter        internal (approximates triangle:2)\n"
 			"\n"
 			);
@@ -469,12 +451,17 @@ public:
 			HEADER_SPACE
 			"--exp-register    Register exposure between frames.  [default]\n"
 			"--exp-noregister  Assume uniform exposure across all frames.\n"
-			"--exp-meta-only   Use only image metadata for registering exposure.\n"
+			"--exp-meta-only   Use only meta-data for registering exposure.\n"
 			BETWEEN_SECTIONS
 			"Range extension:\n"
 			HEADER_SPACE
 			"--exp-extend      Extend to include all calculated values.\n"
 			"--exp-noextend    Restrict to the original frame's range.  [default]\n"
+			BETWEEN_SECTIONS
+			"Exposure value meta-data:\n"
+			HEADER_SPACE
+			"--ev <x>          Set ISO 100 equivalent EV <x>.  (default is 0)\n"
+			"--black <x>       Set black level <x> as a fraction of saturation.  (default 0)\n"
 			"\n"
 		       );
 	}
@@ -544,7 +531,7 @@ public:
 			"--visp <args>     Process a video sequence.\n"
 			"                     <args> are:\n"
 			"                         <chain> <stabilization-type> <prefix> <suffix>\n"
-			"                     <chain> is an incremental rendering chain.  (see --hl)\n"
+			"                     <chain> is a rendering chain.  (see --hl)\n"
 			"                     <stabilization-type> is one of:\n"
 			"                         ma:<x>     Moving average over 2*<x> + 1 frames\n"
 			"                         sf:<x>     Stabilize to single frame number <x>\n"
@@ -562,8 +549,10 @@ public:
 			"User Interfaces:\n"
 			HEADER_SPACE
 			"--ui=<type>       Set user interface to <type>, one of:\n"
+			"                     quiet\n"
 			"                     stream\n"
 			"                     tty [default]\n"
+			"                     log\n"
 #ifndef USE_IOCTL
 			"\n"
 			"                     NOTE: since ALE was compiled without terminal size check,\n"
@@ -682,20 +671,22 @@ public:
 			BETWEEN_SECTIONS
 			"List of arguments admitting scoping:\n"
 			HEADER_SPACE
-			"--mc              (see --ha for more details)\n"
-			"--mcd-removal     (see --ha for more details)\n"
+			"--gs              (see --ha for more details)\n"
+			"--gs-mo           (see --ha for more details)\n"
 			"--threads         (see --hp for more details)\n"
 			"--per-cpu         (see --hp for more details)\n"
 			"--perturb-upper   (see --ha for more details)\n"
+			"--ev              (see --hx for more details)\n"
+			"--black           (see --hx for more details)\n"
 			BETWEEN_SECTIONS
 			"Implicit file scope (implementation may be buggy):\n"
 			HEADER_SPACE
 			"arg1 file1 arg2   <arg2> applies only to files later than <file1>.\n"
 			"                    Example:\n"
 			"\n"
-			"                         --mc 10 file1 --mc 20 file2\n"
+			"                         --ev 10 file1 --ev 20 file2\n"
 			"\n"
-			"                    (file1 has monte-carlo argument 10; file2 has --mc 20.)\n"
+			"                    (file1 has EV 10; file2 has EV 20.)\n"
 			"\n"
 			BETWEEN_SECTIONS
 			"Hidden scope (implementation may be buggy):\n"
@@ -703,9 +694,9 @@ public:
 			"[ args ]          Hide the effects of <args> options within [ ... ] scope\n"
 			"                    Example:\n"
 			"\n"
-			"                         file1 [ --mc 20 file2 ] file3\n"
+			"                         file1 [ --ev 20 file2 ] file3\n"
 			"\n"
-			"                    (The monte-carlo argument is limited to file2.)\n"
+			"                    (The EV argument is limited to file2.)\n"
 			"\n"
 			BETWEEN_SECTIONS
 			"Exposed scope (implementation may be buggy):\n"
