@@ -5,7 +5,7 @@
 
     The Anti-Lamenessing Engine is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
+    the Free Software Foundation; either version 3 of the License, or
     (at your option) any later version.
 
     The Anti-Lamenessing Engine is distributed in the hope that it will be useful,
@@ -30,10 +30,12 @@
 #include <stdlib.h>
 #include <assert.h>
 #include <time.h>
-#include <math.h>
+// #include <math.h>
 #include <stack>
 #include <map>
 
+#include "../ale_math.h"
+
 /*
  * Interface files
  */
@@ -55,7 +57,7 @@
  */
 
 extern "C" {
-#include "string_.h"
+#include "string.h"
 }
 
 /*
@@ -369,6 +371,8 @@ class input {
 			snprintf(desired_string, length, "%u %s", arg, name + strlen("0 "));
 
 			environment_map.erase(desired_string);
+
+			free(desired_string);
 		}
 
 		const char *get_string_arg(const char *name, unsigned int arg) {
@@ -1371,7 +1375,7 @@ public:
 		const char *device = NULL;
 		int psf_match = 0;
 		double psf_match_args[6];
-		int inc = 1;
+		int inc = 0;
 		int exposure_register = 1;
 		const char *wm_filename = NULL;
 		int wm_offsetx = 0, wm_offsety = 0;
@@ -1419,50 +1423,26 @@ public:
 		 * Handle default settings
 		 */
 
-		if (arg_prefix_count(argc, argv, "--q") > 1)
-			ui::get()->error("more than one default setting option --q* was specified");
+		if (arg_prefix_count(argc, argv, "--q") > 0)
+			ui::get()->error("Default settings --q* are no longer recognized.");
 
-		const char *defaults[] = {
-   			"--dchain fine:triangle:2,triangle:2 "
+#define FIXED16 4
+#if ALE_COLORS == FIXED16
+		const char *defaults = 
+   			"--dchain auto:triangle:2,fine:box:1,triangle:2 "
 			"--achain triangle:2 "
 			"--ips 0 "
-			"--3d-chain fine:triangle:2,triangle:2 ",
-
-			"--dchain sinc*lanc:6 "
-			"--achain sinc*lanc:6 "
-			"--ips 0 "
-			"--3d-chain sinc*lanc:6 ",
-   
-			"--dchain fine:triangle:2,fine:gauss:0.75,triangle:2 "
+			"--3d-chain fine:triangle:2,fine:gauss:0.75,triangle:2 ";
+#else
+		const char *defaults = 
+   			"--dchain auto:triangle:2,fine:box:1,triangle:2 "
 			"--achain triangle:2 "
-			"--ips 0 "
-			"--3d-chain fine:triangle:2,fine:gauss:0.75,triangle:2 ",
+			"--ips 1 "
+			"--3d-chain fine:triangle:2,fine:gauss:0.75,triangle:2 ";
+#endif
+#undef FIXED16
 
-			"--dchain triangle:2 "
-			"--achain triangle:2 "
-			"--ips 4 "
-			"--3d-chain fine:triangle:2,fine:gauss:0.75,triangle:2 ",
-		};
-
-		int default_index;
-		if (arg_count(argc, argv, "--q0")) {
-			default_index = 0;
-		} else if (arg_count(argc, argv, "--qn")) {
-			default_index = 1;
-		} else if (arg_count(argc, argv, "--q1")) {
-			default_index = 2;
-		} else if (arg_count(argc, argv, "--q2")) {
-			default_index = 3;
-		} else if (arg_count(argc, argv, "--qr")) {
-			default_index = 4;
-		} else {
-			/*
-			 * Same as --q2
-			 */
-			default_index = 3;
-		}
-
-		token_reader *default_reader = new cstring_token_reader(defaults[default_index]);
+		token_reader *default_reader = new cstring_token_reader(defaults);
 
 		evaluate_stream(default_reader, NULL);
 
@@ -1622,10 +1602,6 @@ public:
 					d3::scene::oc();
 				else 
 					d3::scene::no_oc();
-			} else if (!strcmp(option_name, "gs")) {
-				d2::align::gs(env->get_string_arg(i->first, 1));
-			} else if (!strcmp(option_name, "gs-mo")) {
-				d2::align::gs_mo(env->get_unsigned_arg(i->first, 1));
 			} else if (!strcmp(option_name, "focus")) {
 
 				double one = +1;
@@ -1883,8 +1859,15 @@ public:
 					ui::set_stream();
 				else if (!strcmp(env->get_string_arg(i->first, 1), "tty"))
 					ui::set_tty();
-				else
-					assert(0);
+				else if (!strcmp(env->get_string_arg(i->first, 1), "log"))
+					ui::set_log();
+				else if (!strcmp(env->get_string_arg(i->first, 1), "quiet"))
+					ui::set_quiet();
+				else {
+					fprintf(stderr, "Error: Unknown user interface type '%s'\n", 
+							env->get_string_arg(i->first, 1));
+					exit(1);
+				}
 			} else if (!strcmp(option_name, "3d-fmr")) {
 				d3::scene::fmr(env->get_double_arg(i->first, 1));
 			} else if (!strcmp(option_name, "3d-dmr")) {
@@ -2116,6 +2099,11 @@ public:
 				cx_parameter = env->get_int_arg(i->first, 0) ? env->get_double_arg(i->first, 1) : 0;
 			} else if (!strcmp(option_name, "ip")) {
 				unsupported::discontinued("--ip <r> <i>", "--lpsf box=<r> --ips <i>");
+			} else if (!strcmp(option_name, "cache")) {
+				double cache = env->get_double_arg(i->first, 1);
+
+				d2::image_rw::set_cache(cache);
+
 			} else if (!strcmp(option_name, "bayer")) {
 
 				/*
@@ -2184,9 +2172,9 @@ public:
 				else
 					d3::align::vo_noadjust();
 			} else if (!strcmp(option_name, "ip-statistic")) {
-				if (!strcmp(env->get_string_arg(i->first, 1), "mean"))
+				if (!strcmp(env->get_string_arg(i->first, 0), "mean"))
 					ip_use_median = 0;
-				else if (!strcmp(env->get_string_arg(i->first, 1), "median"))
+				else if (!strcmp(env->get_string_arg(i->first, 0), "median"))
 					ip_use_median = 1;
 			} else if (!strcmp(option_name, "ips")) {
 				ip_iterations = env->get_int_arg(i->first, 1);
@@ -2311,6 +2299,8 @@ public:
 					d3::cpf::set_cpp_lower(perturb_lower);
 			} else if (!strcmp(option_name, "hf-enhance")) {
 				unsupported::discontinued("--hf-enhance=<x>");
+			} else if (!strcmp(option_name, "gs")) {
+				d2::align::gs(env->get_string_arg(i->first, 1));
 			} else if (!strcmp(option_name, "rot-upper")) {
 				d2::align::set_rot_max((int) floor(env->get_double_arg(i->first, 1)));
 			} else if (!strcmp(option_name, "bda-mult")) {
@@ -2375,6 +2365,7 @@ public:
 		}
 #endif
 
+#if 0
 		if (cx_parameter != 0 && !exposure_register) {
 			implication::changed("Certainty-based rendering requires exposure registration.",
 					     "Exposure registration is now enabled.",
@@ -2382,6 +2373,7 @@ public:
 			d2::align::exp_register();
 			exposure_register = 1;
 		}
+#endif
 
 		/*
 		 * Set alignment class exclusion region static variables
@@ -2611,24 +2603,11 @@ public:
 			d2::align::set_weight_map(weight_map);
 		}
 
-		/*
-		 * Write comment information about original frame and
-		 * target image to the transformation save file, if we
-		 * have one.
-		 */
-
-		const d2::image *im = d2::image_rw::open(0);
-		// tsave_orig(tsave, argv[i], im->avg_channel_magnitude());
-		// tsave_target(tsave, argv[argc - 1]);
-		tsave_orig(tsave, files[0].first, im->avg_channel_magnitude());
-		tsave_target(tsave, files[files.size() - 1].first);
-		d2::image_rw::close(0);
-
 		/*
 		 * Initialize alignment interpolant.
 		 */
 
-		if (afilter_type != "internal")
+		if (strcmp(afilter_type, "internal"))
 			d2::align::set_interpolant(d2::render_parse::get_SSF(afilter_type));
 
 		/*
@@ -2731,11 +2710,32 @@ public:
 				const char *option_name = env->get_option_name(i->first);
 
 				if (!strcmp(option_name, "mc")) {
-					int type = env->get_int_arg(i->first, 0);
-					d2::align::mc((type == 2) ? env->get_double_arg(i->first, 1) / 100 
-							          : (double) type);
-				} else if (!strcmp(option_name, "mcd-removal")) {
-					d2::align::mcd_limit(env->get_int_arg(i->first, 1));
+					d2::align::mc(env->get_double_arg(i->first, 1));
+				} else if (!strcmp(option_name, "md")) {
+					d2::align::ma_decomp(env->get_int_arg(i->first, 1));
+				} else if (!strcmp(option_name, "gs-mo")) {
+					const char *option = env->get_string_arg(i->first, 1);
+					double gs_mo;
+					int characters;
+					sscanf(option, "%lf%n", &gs_mo, &characters);
+					if (*(option + characters) == '%')
+						d2::align::gs_mo(gs_mo, 1);
+					else
+						d2::align::gs_mo(gs_mo, 0);
+				} else if (!strcmp(option_name, "ev")) {
+					double ev = env->get_double_arg(i->first, 1);
+					double gain_value = pow(2, -ev);
+
+					if (j == 0)
+						d2::exposure::set_gain_reference(gain_value);
+					else 
+						input_exposure[j]->set_gain_multiplier(
+								(double) d2::exposure::get_gain_reference()
+							      / gain_value);
+
+				} else if (!strcmp(option_name, "black")) {
+					double black = env->get_double_arg(i->first, 1);
+					input_exposure[j]->set_black_level(black);
 				} else if (!strcmp(option_name, "perturb-upper")) {
 					const char *option = env->get_string_arg(i->first, 1);
 					double perturb_upper;
@@ -2766,6 +2766,20 @@ public:
 
 
 			if (j == 0) {
+
+				/*
+				 * Write comment information about original frame and
+				 * target image to the transformation save file, if we
+				 * have one.
+				 */
+
+				const d2::image *im = d2::image_rw::open(0);
+				// tsave_orig(tsave, argv[i], im->avg_channel_magnitude());
+				// tsave_target(tsave, argv[argc - 1]);
+				tsave_orig(tsave, files[0].first, im->avg_channel_magnitude());
+				tsave_target(tsave, files[files.size() - 1].first);
+				d2::image_rw::close(0);
+
 				/*
 				 * Handle the original frame.
 				 */
@@ -2829,14 +2843,16 @@ public:
 		/*
 		 * Do any post-processing and output final image
 		 *
-		 * XXX: note that non-incremental renderers currently
-		 * return zero for ochain[0]->sync(), since they write
+		 * XXX: note that the Irani-Peleg renderer currently
+		 * returns zero for ochain[0]->sync(), since it writes
 		 * output internally when inc != 0.
 		 */
 
 		for (int opt = 0; opt < oc_count; opt++) 
-		if  ((ochain[opt]->sync() || !inc) && !psf_match)
+		if  ((ochain[opt]->sync() || !inc) && !psf_match) {
+			ui::get()->writing_output(opt);
 			d2::image_rw::write_image(ochain_names[opt], ochain[opt]->get_image());
+		}
 
 		/*
 		 * Output a summary match statistic.
@@ -2856,7 +2872,7 @@ public:
 
 			d3::align::init_angle(view_angle);
 
-			ui::get()->d3_init_view_angle(view_angle / M_PI * 180);
+			ui::get()->d3_init_view_angle((double) view_angle / M_PI * 180);
 
 			d3::align::init_from_d2();
 
